<?php

	/*
	*	Gekko - Open Source Web Development Framework
	*	------------------------------------------------------------------------
	*	Copyright (C) 2004-2006, J. Carlos Nieto <xiam@users.sourceforge.net>
	*	This program is Free Software.
	*
	*	@package	Gekko
	*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
	*	@author		J. Carlos Nieto <xiam@users.sourceforge.net>
	*	@link		http://www.gekkoware.org
	*/

	define ("IN-GEKKO", true);

	require("../../conf.php");

	LoadLib (
		"file.lib.php", "image.lib.php", "gallery"
	);

	require_once GEKKO_SOURCE_DIR."lib/auth.inc.php";

	varImport("action", "return", "id");

	if (is_admin()) {
		switch ($action) {
			case "delete_item":
				execWithEach($id, "gallery::deleteItem");
			break;
			case "delete_gallery":
				execWithEach($id, "gallery::deleteGallery");
			break;
			case "enable_gallery":
				gallery::setGalleryStatus($id, "1");
			break;
			case "disable_gallery":
				gallery::setGalleryStatus($id, "0");
			break;
			case "edit_gallery":

				varImport("id", "status", "gallery_id", "title", "description", "auth_access");

				varRequire("id", "title");

				// a thumbnail is required
				if ($_POST["remote_file"] || $_FILES["local_file"]["size"]) {
					$thumb = file_get("local_file", "remote_file", GEKKO_GALLERY_REL_DATADIR);
					if ($thumb) {
						$thumb = mkthumbnail($thumb["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"));
						$icon = $thumb["url"];
					} else {
						formInputError("{E_DOWNLOAD_ERROR}", "focus: remote_file");
					}
					// deleting old thumbnail_url
					$q = $db->query("SELECT thumbnail_url
					FROM ".db_table("gallery")." WHERE id = '{$id}'", __FILE__, __LINE__);
					$row = $db->fetchRow($q);
					@unlink(GEKKO_USER_DIR.$row["thumbnail_url"]);
				} else {
					$thumb = false;
				}

				editorApplyFixes($description);
				$db->query("UPDATE ".db_table("gallery")." SET
					title = '{$title}',
					description = '{$description}',
					gallery_id = '{$gallery_id}',
					status = '{$status}',
					author_id = '{$USER["id"]}',
					".($thumb ? "thumbnail_url = '{$thumb["url"]}'," : "")."
					".($thumb ? "thumbnail_size = '{$thumb["width"]}x{$thumb["height"]}'," : "")."
					auth_access = '".groups::extractFromInput($auth_access)."'
				WHERE id = '{$id}'
				", __FILE__, __LINE__);

				slog("gallery: gallery '$id' was updated", "actions", 1);
			break;
			case "create_gallery":

				varImport("status", "gallery_id", "title", "description", "auth_access");

				// thumbnail is required
				if ($_POST["remote_file"] || $_FILES["local_file"]["size"]) {
					$thumb = file_get("local_file", "remote_file", GEKKO_GALLERY_REL_DATADIR);
					if ($thumb) {
						$thumb = mkthumbnail($thumb["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"));
					} else {
						abort("Couldn't get thumbnail.");
					}
				} else {
					formInputError("{E_INSUFFICIENT_DATA}", "focus: remote_file");
				}

				if (!$title)
					$title = beauty_basename($thumb["origin"]);

				editorApplyFixes($description);
				$db->query ("
				INSERT INTO ".db_table("gallery")."
					(
						id,
						gallery_id,
						title,
						description,
						status,
						thumbnail_url,
						thumbnail_size,
						author_id,
						date_created,
						auth_access
					)
						VALUES
					(
						'".$db->findFreeId("gallery")."',
						'{$gallery_id}',
						'{$title}',
						'{$description}',
						'{$status}',
						'{$thumb["url"]}',
						'{$thumb["width"]}x{$thumb["height"]}',
						'{$USER["id"]}',
						'".time2date()."',
						'".groups::extractFromInput($auth_access)."'
					)
				", __FILE__, __LINE__);

				slog("gallery: new gallery '$title'", "actions", 2);
			break;
			case "edit_item":

				varImport("id", "gallery_id", "title", "description", "link_only",
				"media_url", "thumbnail_url", "allow_comments");

				varRequire("id", "title", "gallery_id");

				// changed media
				$media = Array();
				if ((!$link_only) && ($_POST["remote_media"] || $_FILES["local_media"]["size"])) {
					$media = file_get("local_media", "remote_media", GEKKO_GALLERY_REL_DATADIR);
					if (!file_exists($media["abs_loc"])) {
						formInputError("{E_DOWNLOAD_ERROR}", "focus: remote_media");
					}
				} elseif ($link_only && $_POST["remote_media"]) {
					// only linking
					$media["rel_loc"] = $_POST["remote_media"];
					if (!$title)
						$title = beauty_basename($media);

					// checking if thumnbnail was provided
					if (!$_POST["remote_file"] && !$_FILES["local_file"]["size"]) {
						// thumbnail was not provided, but if media is an image, so we can try to make a copy in thumbnail size
						switch (gallery::guessMediaType($media["rel_loc"])) {
							case "image":
								$_POST["remote_file"] = $media["rel_loc"];
							break;
						}
					}
				}

				// changed thumbnail_url
				$thumb = Array();
				if ($_POST["remote_file"] || $_FILES["local_file"]["size"]) {
					$thumb = file_get("local_file", "remote_file", GEKKO_GALLERY_REL_DATADIR);
					if ($thumb) {
						if ($link_only) {
							// saving image size before resizing
							$media["size"] = imagesize($thumb["abs_loc"]);
						}
						$thumb = mkthumbnail($thumb["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"));
					}
				}

				if ($media && !$thumb) {
					// no thumbnail was provided, media was changed, trying to update...
					switch (file_extension($media["basename"])) {
						case "jpg": case "jpeg": case "png": case "gif": case "xpm": case "tiff":
							$thumb = mkthumbnail($media["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"), false, false);
						break;
						default:
							abort(_l("E_COULDNT_CREATE_THUMBNAIL"));
						break;
					}
				}

				if ($media || $thumb) {
					// deleting old files
					$q = $db->query("SELECT thumbnail_url, media_url FROM ".db_table("gallery_item")."
					WHERE id = '{$id}'", __FILE__, __LINE__);
					$row = $db->fetchRow($q);
					if ($media)
						@unlink(GEKKO_USER_DIR.$row["media_url"]);
					if ($thumb)
						@unlink(GEKKO_USER_DIR.$row["thumbnail_url"]);
				}

				// default media size
				if ($media) {
					// default media size
					if (!isset($media["size"])) {
						$media["size"] = isset($media["abs_loc"]) ? imagesize($media["abs_loc"]) : conf::getkey("gallery", "default_size");
					}
				}

				editorApplyFixes($description);

				categories::tagUpdate("gallery", $id, $tags, array($description, $title));

				$db->query("UPDATE ".db_table("gallery_item")." SET
					gallery_id = '{$gallery_id}',
					title = '{$title}',
					description = '{$description}',
					thumbnail_url = '".(isset($thumb["url"]) ? $thumb["url"] : $thumbnail_url)."',
					".($thumb ? "thumbnail_size = '{$thumb["width"]}x{$thumb["height"]}'," : "")."
					media_url = '".(isset($media["rel_loc"]) ? $media["rel_loc"] : $media_url)."',
					".($media ? "media_size = '{$media["size"]}'," : "")."
					author_id = '{$USER["id"]}',
					allow_comments = '{$allow_comments}'
				WHERE id = '{$id}'", __FILE__, __LINE__);

				slog("gallery: item '$id' was updated", "actions", 1);
			break;
			case "import_items":

				LoadLib ("compression.lib.php");

				$rel_workdir = "temp/gallery";
				$abs_workdir = GEKKO_USER_DIR.$rel_workdir;

				varImport("gallery_id", "description", "allow_comments", "remote_media", "tags");

				varRequire("gallery_id");

				clean_dir($abs_workdir);

				if ($_POST["remote_package"] || $_FILES["local_package"]["size"]) {

					$package = file_get("local_package", "remote_package", $rel_workdir);

					if (file_exists($package["abs_loc"])) {

						// extracting file to a temporary directory
						package_extract($package["abs_loc"], $abs_workdir);

						// deleting compressed file
						unlink($package["abs_loc"]);

						// listing package contents
						$contents = listdir($abs_workdir, true);

						$media_files = Array();
						foreach ($contents as $media) {
							if (is_file("$abs_workdir/$media")) {

								// avoiding thumbnails/ directory
								if (!preg_match("/(^thumbnails\/|\/thumbnails\/).*/", $media)) {
									// does this media file includes its own thumbnail.
									if (!file_exists("$abs_workdir/".dirname($media)."/thumbnails/$media")) {
										$abs_media = "$abs_workdir/$media";
										// if not, creating one
										switch (gallery::guessMediaType($abs_media)) {
											case "image":
												$thumb = mkthumbnail($abs_media, conf::getKey("gallery", "thumbnail.longest_side"), false, false, "$rel_workdir/".dirname($media)."/thumbnails");
											break;
											default:
												$thumb = mkGenericThumbnail($abs_media, conf::getKey("gallery", "thumbnail.longest_side"), "$rel_workdir/".dirname($media)."/thumbnails");
											break;
										}
									}
									$media_files[] = array("file" => $media, "thumb" => $thumb);
								}
							}
						}

						// there were no errors, proceding to move to data/gallery and inserting
						// media file into database

						foreach ($media_files as $media_file) {

							extract($media_file);

							// allocating name for media file
							$s_media = "$abs_workdir/$file";
							$d_media = "data/gallery/".allocate_basename(GEKKO_USER_DIR."data/gallery", basename($s_media));

							// allocating name for thumbnail_url
							$s_thumb = $thumb["path"];
							$d_thumb = "data/thumbs/".allocate_basename(GEKKO_USER_DIR."data/thumbs", basename($s_thumb));

							// moving files
							rename($s_media, GEKKO_USER_DIR.$d_media);
							rename($s_thumb, GEKKO_USER_DIR.$d_thumb);

							// getting size
							if (($media_size = imagesize(GEKKO_USER_DIR.$d_media)) == false)
								$media_size = conf::getkey("gallery", "default_size");

							if (($thumb_size = imagesize(GEKKO_USER_DIR.$d_thumb)) == false)
								$thumb_size = "100x75";

							$title = beauty_basename($file);
							editorApplyFixes($description);

							$item_id = $db->findFreeId("gallery_item");

							categories::tagUpdate("gallery", $item_id, $tags, array($description, $title));

							// inserting item
							$db->query ("
							INSERT INTO ".db_table("gallery_item")."
								(
									id,
									gallery_id,
									title,
									description,
									thumbnail_url,
									thumbnail_size,
									media_url,
									media_size,
									author_id,
									date_created,
									allow_comments
								)
									VALUES
								(
									'{$item_id}',
									'{$gallery_id}',
									'{$title}',
									'{$description}',
									'{$d_thumb}',
									'{$thumb_size}',
									'{$d_media}',
									'{$media_size}',
									'{$USER["id"]}',
									'".time2date()."',
									'{$allow_comments}'
								)
							", __FILE__, __LINE__);

						}

					} else {
						trigger_error("Couldn't get package!", E_USER_ERROR);
					}
				} else {
					trigger_error("Couldn't upload package!", E_USER_ERROR);
				}

			break;
			case "create_item":

				unset($size);

				varImport("gallery_id", "title", "description", "link_only", "allow_comments", "tags");

				varRequire("gallery_id");

				$media = Array();

				// media is required
				if ((!$link_only) && ($_POST["remote_media"] || $_FILES["local_media"]["size"])) {
					// saving media
					$media = file_get("local_media", "remote_media", GEKKO_GALLERY_REL_DATADIR);
					if (!file_exists($media["abs_loc"])) {
						abort("Couldn't upload media.");
					}
					if (!$title)
						$title = beauty_basename($media["basename"]);
				} elseif ($link_only && $_POST["remote_media"]) {
					// only linking
					$media["rel_loc"] = $_POST["remote_media"];
					if (!$title)
						$title = beauty_basename($media);
					// checking if thumnbnail was provided
					if (!$_POST["remote_file"] && !$_FILES["local_file"]["size"]) {
						// thumbnail was not provided, but if media is an image, so we can try to make a copy in thumbnail.size
						switch (gallery::guessMediaType($media["rel_loc"])) {
							case "image":
								$_POST["remote_file"] = $media["rel_loc"];
							break;
						}
					}
				} else {
					abort(_l("E_INSUFFICIENT_DATA"));
				}

				// custom thumbnail_url
				if ($_POST["remote_file"] || $_FILES["local_file"]["size"]) {
					$thumb = file_get("local_file", "remote_file", GEKKO_GALLERY_REL_DATADIR);
					if ($thumb) {
						if ($link_only) {
							// saving image size before resizing
							$media["size"] = imagesize($thumb["abs_loc"]);
						}
						$thumb = mkthumbnail($thumb["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"));
					}
				} elseif (!$link_only) {
					// creating thumbnail taking media as base (when possible)
					switch (gallery::guessMediaType($media["basename"])) {
						case "image":
							// we can create a thumbnail with the uploaded image
						break;
						default:
							// media is not an image! no thumbnail was provided... hmm!
							unlink($media["abs_loc"]);
							abort(_l("E_COULDNT_CREATE_THUMBNAIL"));
						break;
					}
					$thumb = mkthumbnail($media["abs_loc"], conf::getKey("gallery", "thumbnail.longest_side"), false, false);
				} else {
					abort(_l("E_COULDNT_CREATE_THUMBNAIL"));
				}

				if (!isset($thumb) || !file_exists($thumb["path"])) {
					unlink($media["abs_loc"]);
					abort(_l("E_COULDNT_CREATE_THUMBNAIL"));
				}

				// default media size
				if (!isset($media["size"]))
					$media["size"] = isset($media["abs_loc"]) ? imagesize($media["abs_loc"]) : conf::getkey("gallery", "default_size");

				editorApplyFixes($description);

				$item_id = $db->findFreeId("gallery_item");

				categories::tagUpdate("gallery", $item_id, $tags, array($description, $title));

				$db->query ("
				INSERT INTO ".db_table("gallery_item")."
					(
						id,
						gallery_id,
						title,
						description,
						thumbnail_url,
						thumbnail_size,
						media_url,
						media_size,
						author_id,
						date_created,
						allow_comments
					)
						VALUES
					(
						'{$item_id}',
						'{$gallery_id}',
						'{$title}',
						'{$description}',
						'{$thumb["url"]}',
						'{$thumb["width"]}x{$thumb["height"]}',
						'{$media["rel_loc"]}',
						'{$media["size"]}',
						'{$USER["id"]}',
						'".time2date()."',
						'{$allow_comments}'
					)
				", __FILE__, __LINE__);

				slog("gallery: added item to gallery '$title'", "actions", 2);
			break;
		}
	}

	redirect($return);

	db_exit();
?>